// MulticastClient.java

import java.io.*;
import java.net.*;

public class MulticastClient {

  public static void main(String[] args) {
    InetAddress addr = null;
    int port = -1;
    try{
      if(args.length == 2){
        addr = InetAddress.getByName(args[0]);
        port = Integer.parseInt(args[1]);
        System.out.println("Mi pongo in ascolto su:\nMCastAddr: " + args[0]
            + "\nMCastPort: " + port);
      }
      else{
        System.out.println("Usage: java MulticastClient MCastAddr MCastPort");
        System.exit(1);
      }
    } //try
    catch(Exception e){
      System.out.println("Problemi, i seguenti: ");
      e.printStackTrace();
      System.exit(1);
    }

    // creazione della socket multicast
    MulticastSocket socket = null;
    byte[] buf = new byte[256];
    DatagramPacket packet = new DatagramPacket(buf, buf.length);

    try{
      socket = new MulticastSocket(port);
      socket.setSoTimeout(20000); // 20 secondi
      System.out.println("\nMulticastClient: avviato");
      System.out.println("Creata la socket: " + socket);
    }
    catch(IOException e){
      System.out.println("Problemi nella creazione della socket: ");
      e.printStackTrace();
      System.exit(2);
    }

    // adesione al gruppo associato all'indirizzo multicast
    try{
      /* Per evitare problemi, con computer non connessi in rete,
       * bisogna impostare l'interfaccia di rete PRIMA di fare la
       * join al gruppo.
       *
       * VEDERE ANCHE LE FAQ nel sito!!!
       */
      // mi procuro l'inetaddress locale
      //InetAddress netInterface= InetAddress.getByName("localhost");
      // imposto l'interfaccia di rete
      //socket.setInterface(netInterface);
      socket.joinGroup(addr);
      System.out.println("Adesione al gruppo " + addr);
    }
    catch(IOException e){
      System.out.println("Problemi nell'adesione al gruppo: ");
      e.printStackTrace();
      System.exit(3);
    }

    ByteArrayInputStream biStream = null;
    DataInputStream diStream = null;
    String linea = null;

    // ricezione di alcune linee
    for(int i = 0; i < 20; i++){

      System.out.println("\nIn attesa di un datagramma... ");
      try{
        packet.setData(buf);
        socket.receive(packet);
      }
      catch(SocketTimeoutException te){
        System.out.println("Non ho ricevuto niente per 20 secondi, chiudo!");
        System.exit(4);
      }
      catch(IOException e){
        System.out.println("Problemi nella ricezione del datagramma: ");
        e.printStackTrace();
        // anche se ci sono problemi riprende il ciclo di ricezioni
        continue;
      }
      try{
        biStream = new ByteArrayInputStream(packet.getData(), 0, packet
            .getLength());
        diStream = new DataInputStream(biStream);
        linea = diStream.readUTF();
        System.out.println("Linea ricevuta: " + linea);
      }
      catch(IOException e){
        System.out.println("Problemi nella lettura del datagramma: ");
        e.printStackTrace();
        continue;
        // anche se ci sono problemi riprende il ciclo di ricezioni
      }

    } //for

    // uscita dal gruppo e chiusura della socket
    System.out.println("\nUscita dal gruppo");
    try{
      socket.leaveGroup(addr);
    }
    catch(IOException e){
      System.out.println("Problemi nell'uscita dal gruppo: ");
      e.printStackTrace();
    }

    System.out.println("MulticastClient: termino...");
    socket.close();
  }

}